<?
/***********************************************************************

  Copyright (C) 2007  Yaraba (rokmcssu@cubrid.com)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


// Make sure we have built in support for CUBRID
if (!function_exists('cubrid_connect'))
	exit('This PHP environment doesn\'t have CUBRID support built in. CUBRID support is required if you want to use a CUBRID database to run this forum. Consult the PHP documentation for further assistance.');


class DBLayer
{
	var $prefix;
	var $link_id;
	var $query_result;
	var $last_query_text = array();
	var $in_transaction = 0;

	var $saved_queries = array();
	var $num_queries = 0;
    var $last_insertid = 0;

	var $deli = "\"";
	var $error_no = false;
	var $error_msg = 'Unknown';


	function DBLayer($db_host, $db_username, $db_password, $db_name, $db_prefix, $p_connect)
	{
		$this->prefix = $db_prefix;

		if ($db_host != '')
		{
			if (strpos($db_host, ':') !== false)
			{
				list($db_host, $dbport) = explode(':', $db_host);
			}
			else
			{
				$dbport = 33000;
			}
		}
		else {
			$db_host = 'localhost';
			$dbport = 33000;
		} 
			

		$this->link_id = @cubrid_connect($db_host, $dbport, $db_name, $db_username, $db_password);

		if (!$this->link_id)
			error('Unable to connect to CUBRID server.', __FILE__, __LINE__);
		else
			return $this->link_id;
	}

    function limit_make($from, $cnt=0)
    {
        if ($cnt<=0) {
            if ($from == 1)
                return " for orderby_num() = 1";
            else 
                return " for orderby_num() BETWEEN 1 AND $from";
        }
        else {
            return " for orderby_num() BETWEEN ".($from+1)." AND ".($from+$cnt);
        }
    }
    
	function start_transaction()
	{
		++$this->in_transaction;

		return true;
	}


	function end_transaction()
	{
		--$this->in_transaction;

		if (@cubrid_commit($this->link_id))
			return true;
		else
		{
			@cubrid_rollback($this->link_id);
			return false;
		}
	}

	function query($sql, $unbuffered = false)	// $unbuffered is ignored since there is no pgsql_unbuffered_query()
	{
        global $debug_cubrid_sql;
        
		if (defined('PUN_SHOW_QUERIES'))
			$q_start = get_microtime();

		if (preg_match('/^INSERT INTO ([a-z0-9\_\-]+)/is', $sql, $table_name))
		{
            $tbl = substr($table_name[1], strlen($this->prefix));
            if ($tbl != "config" && $tbl != "forum_perms" && $tbl != "online" && $tbl != "search_cache" && 
                $tbl != "search_matches" && $tbl != "subscriptions" && $tbl != "serialbase") { // no auto_increment table
    			$temp_q_id = @cubrid_execute($this->link_id, 'SELECT '.$table_name[1].'_seq.next_value from '.$this->prefix.'serialbase');
                $this->last_insertid = false;
    			if ($temp_q_rst = @cubrid_fetch($temp_q_id, CUBRID_NUM)) {
    			    $this->last_insertid = intval($temp_q_rst[0]);
                }
                // INSERT serial
                $inc_attr=($tbl=="groups") ? "g_id" : "id";
                $stapos=strpos($sql, "INSERT INTO");
                $stapos += 11;
                $stapos=strpos($sql, "(", $stapos);
                $sql = substr($sql,0,$stapos+1)."$inc_attr ,".substr($sql,$stapos+1);
                $stapos += 5;
                $stapos=strpos($sql, "VALUES", $stapos);
                $stapos += 6;
                $stapos=strpos($sql, "(", $stapos);
                $sql = substr($sql,0,$stapos+1).$this->last_insertid." ,".substr($sql,$stapos+1);
            }
		}

        if ($debug_cubrid_sql) $sql=$sql." /*".$this->backtrace()."*/";
		$this->query_result = @cubrid_execute($this->link_id, $sql);

		if ($this->query_result)
		{
			if (defined('PUN_SHOW_QUERIES'))
				$this->saved_queries[] = array($sql, sprintf('%.5f', get_microtime() - $q_start));

			++$this->num_queries;
			$this->last_query_text[$this->query_result] = $sql;

			return $this->query_result;
		}
		else
		{
			if (defined('PUN_SHOW_QUERIES'))
				$this->saved_queries[] = array($sql, 0);

			return false;
		}
	}

	function result($query_id = 0, $row = 0)
	{
		if ($query_id) {
			cubrid_move_cursor($query_id, $row+1, CUBRID_CURSOR_FIRST);
			$rst=cubrid_fetch($query_id, CUBRID_NUM);
			return $rst[0];
		}
		else
			return false;
	}


	function fetch_assoc($query_id = 0)
	{
		return ($query_id) ? @cubrid_fetch($query_id, CUBRID_ASSOC) : false;
	}


	function fetch_row($query_id = 0)
	{
		return ($query_id) ? @cubrid_fetch($query_id, CUBRID_NUM) : false;
	}


	function num_rows($query_id = 0)
	{
		return ($query_id) ? @cubrid_num_rows($query_id) : false;
	}


	function affected_rows()
	{
		return ($this->query_result) ? @cubrid_affected_rows($this->query_result) : false;
	}


	function insert_id()
	{
		$query_id = $this->query_result;

		if ($query_id && $this->last_query_text[$query_id] != '')
		{
            return $this->last_insertid;
		}

		return false;
	}


	function get_num_queries()
	{
		return $this->num_queries;
	}


	function get_saved_queries()
	{
		return $this->saved_queries;
	}


	function free_result($query_id = false)
	{
		if (!$query_id)
			$query_id = $this->query_result;
		
		return ($query_id) ? @cubrid_close_request($query_id) : false;
	}


	function escape($str)
	{
		if (is_array($str))
			return '';
	    $cutlen=15000;
	    $clen=strlen($str);
	    if ($clen<=$cutlen)
	      return eregi_replace(chr(39), chr(39).chr(39),$str);
	    else {
	      $wrk="";
	      $off=0;
	      while($off<$clen) {
	        $wlen=$clen-$off;
	        if ($wlen>$cutlen) $wlen=$cutlen;
	        if ($off>0) $wrk .= "' + '";
	        $wrk .= eregi_replace(chr(39), chr(39).chr(39), substr($str, $off, $wlen));
	        $off += $wlen;
	      }
	      return $wrk;
	    }
	}


	function error()
	{
		$result['error_sql'] = @current(@end($this->saved_queries));
		$result['error_no'] = @cubrid_error_code();
		$result['error_msg'] = @cubrid_error_msg();

		return $result;
	}


	function close()
	{
		if ($this->link_id)
		{
			if ($this->query_result)
				@cubrid_close_request($this->query_result);

			return @cubrid_disconnect($this->link_id);
		}
		else
			return false;
	}
    
    function backtrace()
    {
        $output = "<div style='text-align: left;'>\n";
        $output .= "<b>Backtrace:</b><br />\n";
        $backtrace = debug_backtrace();

        foreach ($backtrace as $bt) {
            $args = '';
            foreach ($bt['args'] as $a) {
                if (!empty($args)) {
                    $args .= ', ';
                }
                switch (gettype($a)) {
                case 'integer':
                case 'double':
                    $args .= $a;
                    break;
                case 'string':
                    $a = htmlspecialchars(substr($a, 0, 512)).((strlen($a) > 512) ? '...' : '');
                    $args .= "\"$a\"";
                    break;
                case 'array':
                    $args .= 'Array('.count($a).')';
                    break;
                case 'object':
                    $args .= 'Object('.get_class($a).')';
                    break;
                case 'resource':
                    $args .= 'Resource('.strstr($a, '#').')';
                    break;
                case 'boolean':
                    $args .= $a ? 'True' : 'False';
                    break;
                case 'NULL':
                    $args .= 'Null';
                    break;
                default:
                    $args .= 'Unknown';
                }
            }
            $output .= "<br />\n";
            $output .= "<b>file:</b> {$bt['line']} - {$bt['file']}<br />\n";
            $output .= "<b>call:</b> {$bt['class']}{$bt['type']}{$bt['function']}($args)<br />\n";
        }
        $output .= "</div>\n";
        $output = str_replace("/*", "//",$output);
        $output = str_replace("*/", "//",$output);
        return $output;
    }

}
?>